@@ -14,7 +14,7 @@ from deduplipy.config import DEDUPLICATION_ID_NAME, ROW_ID, N_PERFECT_MATCHES_TR
 class Deduplicator:
     def __init__(self, col_names: Optional[List[str]] = None, field_info: Optional[Dict] = None,
                  interaction: bool = False, rules: Union[List[Callable], Dict] = None, recall=1.0,
-                 save_intermediate_steps: bool = False, verbose: Union[int, bool] = 0) -> 'Deduplicator':
+                 save_intermediate_steps: bool = False, verbose: Union[int, bool] = 0):
         """
         Deduplicate entries in Pandas dataframe using columns with names `col_names`. Training takes place during a
         short, interactive session (interactive learning).
@@ -101,7 +101,7 @@ class Deduplicator:
         """
         X_pool = X.copy()
         X_pool[ROW_ID] = np.arange(len(X_pool))
-        df_sample = X_pool.sample(n=int(n_samples ** 0.5))
+        df_sample = X_pool.sample(n=min([len(X_pool), int(n_samples ** 0.5)]))
 
         pairs_table = pd.DataFrame(
             list(product(df_sample[self.col_names + [ROW_ID]].values.tolist(),
@@ -136,7 +136,7 @@ class Deduplicator:
         X.drop(columns=metrics_col_names, inplace=True)
         return X
 
-    def fit(self, X: pd.DataFrame, n_samples: int = 10_000) -> 'Deduplicator':
+    def fit(self, X: pd.DataFrame, n_samples: int = 100_000) -> 'Deduplicator':
         """
         Fit the deduplicator instance
 
@@ -178,7 +178,8 @@ class Deduplicator:
                                                                                     max_cluster_id + 1 + n_missing)
         return X
 
-    def predict(self, X: pd.DataFrame, score_threshold: float = 0.1, cluster_threshold: float = 0.5) -> pd.DataFrame:
+    def predict(self, X: pd.DataFrame, score_threshold: float = 0.1, cluster_threshold: float = 0.5,
+                fill_missing=True) -> pd.DataFrame:
         """
         Predict on new data using the trained deduplicator.
 
@@ -186,6 +187,7 @@ class Deduplicator:
             X: Pandas dataframe with column as used when fitting deduplicator instance
             score_threshold: Classification threshold to use for filtering before starting hierarchical clustering
             cluster_threshold: threshold to apply in hierarchical clustering
+            fill_missing: whether or not to apply missing value imputation on adjacency matrix
 
         Returns: Pandas dataframe with a new column `deduplication_id`. Rows with the same `deduplication_id` are
         deduplicated.
@@ -214,7 +216,7 @@ class Deduplicator:
         if self.save_intermediate_steps:
             scored_pairs_table.to_csv('scored_pairs_table.csv', index=None, sep="|")
         df_clusters = hierarchical_clustering(scored_pairs_table, col_names=self.col_names,
-                                              cluster_threshold=cluster_threshold)
+                                              cluster_threshold=cluster_threshold, fill_missing=fill_missing)
         X = X.merge(df_clusters, on=ROW_ID, how='left').drop(columns=[ROW_ID])
         if self.verbose:
             print('Clustering finished')
